@@ -11,7 +11,6 @@ var { XPCOMUtils } = ChromeUtils.importESModule(
 
 ChromeUtils.defineESModuleGetters(this, {
   BrowserUtils: "resource://gre/modules/BrowserUtils.sys.mjs",
-  Deprecated: "resource://gre/modules/Deprecated.sys.mjs",
   DownloadLastDir: "resource://gre/modules/DownloadLastDir.sys.mjs",
   DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
   Downloads: "resource://gre/modules/Downloads.sys.mjs",
@@ -78,7 +77,8 @@ function saveURL(
   aCookieJarSettings,
   aSourceDocument,
   aIsContentWindowPrivate,
-  aPrincipal
+  aPrincipal,
+  aSaveCompleteCallback
 ) {
   internalSave(
     aURL,
@@ -96,7 +96,8 @@ function saveURL(
     aSkipPrompt,
     null,
     aIsContentWindowPrivate,
-    aPrincipal
+    aPrincipal,
+    aSaveCompleteCallback
   );
 }
 
@@ -120,6 +121,19 @@ function saveBrowser(aBrowser, aSkipPrompt, aBrowsingContext = null) {
         throw new Error("Must have an nsIWebBrowserPersistDocument!");
       }
 
+      // If we are downloading a document that is saving as a URL, we should use
+      // a principal constructed from the document URL. This is important for
+      // documents whose URL is different that their principal, e.g files in the
+      // JSON viewer.
+      let principal = null;
+      if (document.principal?.spec == "resource://devtools/client/jsonview/") {
+        let ssm = Services.scriptSecurityManager;
+        principal = ssm.createContentPrincipal(
+          makeURI(document.documentURI),
+          document.principal.originAttributes
+        );
+      }
+
       internalSave(
         document.documentURI,
         null, // originalURL
@@ -134,7 +148,9 @@ function saveBrowser(aBrowser, aSkipPrompt, aBrowsingContext = null) {
         document.cookieJarSettings,
         document,
         aSkipPrompt,
-        document.cacheKey
+        document.cacheKey,
+        undefined,
+        principal
       );
     },
     onError(status) {
@@ -257,6 +273,8 @@ XPCOMUtils.defineConstant(this, "kSaveAsType_Text", kSaveAsType_Text);
  *        This parameter is provided when neither aDocument nor
  *        aInitiatingDocument is provided. Used to determine what level of
  *        privilege to load the URI with.
+ * @param aSaveCompleteCallback [optional]
+ *        A callback function to call when the save is complete.
  */
 function internalSave(
   aURL,
@@ -274,7 +292,8 @@ function internalSave(
   aSkipPrompt,
   aCacheKey,
   aIsContentWindowPrivate,
-  aPrincipal
+  aPrincipal,
+  aSaveCompleteCallback
 ) {
   if (aSkipPrompt == undefined) {
     aSkipPrompt = false;
@@ -326,11 +345,13 @@ function internalSave(
     };
 
     // Find a URI to use for determining last-downloaded-to directory
-    let relatedURI = aReferrerInfo?.originalReferrer || sourceURI;
+    let relatedURI =
+      aOriginalURL || aReferrerInfo?.originalReferrer || sourceURI;
 
     promiseTargetFile(fpParams, aSkipPrompt, relatedURI)
       .then(aDialogAccepted => {
         if (!aDialogAccepted) {
+          aSaveCompleteCallback?.();
           return;
         }
 
@@ -389,6 +410,7 @@ function internalSave(
       contentPolicyType,
       cookieJarSettings: aCookieJarSettings,
       isPrivate,
+      saveCompleteCallback: aSaveCompleteCallback,
     };
 
     // Start the actual save process
@@ -432,6 +454,8 @@ function internalSave(
  *        If true, the document will always be refetched from the server
  * @param persistArgs.isPrivate
  *        Indicates whether this is taking place in a private browsing context.
+ * @param persistArgs.saveCompleteCallback [optional]
+ *        A callback function to call when the save is complete.
  */
 function internalPersist(persistArgs) {
   var persist = makeWebBrowserPersist();
@@ -467,6 +491,13 @@ function internalPersist(persistArgs) {
     persistArgs.sourceReferrerInfo
   );
   persist.progressListener = new DownloadListener(window, tr);
+  const { saveCompleteCallback } = persistArgs;
+  if (saveCompleteCallback) {
+    tr.downloadPromise
+      .then(aDownload => aDownload.whenSucceeded())
+      .catch(console.error)
+      .finally(saveCompleteCallback);
+  }
 
   if (persistArgs.sourceDocument) {
     // Saving a Document, not a URI:
@@ -522,6 +553,7 @@ function internalPersist(persistArgs) {
  * Structure for holding info about automatically supplied parameters for
  * internalSave(...). This allows parameters to be supplied so the user does not
  * need to be prompted for file info.
+ *
  * @param aFileAutoChosen This is an nsIFile object that has been
  *        pre-determined as the filename for the target to save to
  * @param aUriAutoChosen  This is the nsIURI object for the target
@@ -534,6 +566,7 @@ function AutoChosen(aFileAutoChosen, aUriAutoChosen) {
 /**
  * Structure for holding info about a URL and the target filename it should be
  * saved to. This structure is populated by initFileInfo(...).
+ *
  * @param aSuggestedFileName This is used by initFileInfo(...) when it
  *        cannot 'discover' the filename from the url
  * @param aFileName The target filename
@@ -559,6 +592,7 @@ function FileInfo(
  * Determine what the 'default' filename string is, its file extension and the
  * filename without the extension. This filename is used when prompting the user
  * for confirmation in the file picker dialog.
+ *
  * @param aFI A FileInfo structure into which we'll put the results of this method.
  * @param aURL The String representation of the URL of the document being saved
  * @param aURLCharset The charset of aURL.
@@ -625,6 +659,7 @@ function initFileInfo(
 /**
  * Given the Filepicker Parameters (aFpP), show the file picker dialog,
  * prompting the user to confirm (or change) the fileName.
+ *
  * @param aFpP
  *        A structure (see definition in internalSave(...) method)
  *        containing all the data used within this method.
@@ -688,7 +723,7 @@ function promiseTargetFile(
     let fp = makeFilePicker();
     let titleKey = aFpP.fpTitleKey || "SaveLinkTitle";
     fp.init(
-      window,
+      window.browsingContext,
       ContentAreaUtils.stringBundle.GetStringFromName(titleKey),
       Ci.nsIFilePicker.modeSave
     );
@@ -1149,7 +1184,8 @@ function getCharsetforSave(aDocument) {
 
 /**
  * Open a URL from chrome, determining if we can handle it internally or need to
- *  launch an external application to handle it.
+ * launch an external application to handle it.
+ *
  * @param aURL The URL to be opened
  *
  * WARNING: Please note that openURL() does not perform any content security checks!!!
@@ -1180,10 +1216,10 @@ function openURL(aURL) {
     var appstartup = Services.startup;
 
     var loadListener = {
-      onStartRequest: function ll_start(aRequest) {
+      onStartRequest: function ll_start() {
         appstartup.enterLastWindowClosingSurvivalArea();
       },
-      onStopRequest: function ll_stop(aRequest, aStatusCode) {
+      onStopRequest: function ll_stop() {
         appstartup.exitLastWindowClosingSurvivalArea();
       },
       QueryInterface: ChromeUtils.generateQI([
@@ -1194,13 +1230,13 @@ function openURL(aURL) {
     loadgroup.groupObserver = loadListener;
 
     var uriListener = {
-      doContent(ctype, preferred, request, handler) {
+      doContent() {
         return false;
       },
-      isPreferred(ctype, desired) {
+      isPreferred() {
         return false;
       },
-      canHandleContent(ctype, preferred, desired) {
+      canHandleContent() {
         return false;
       },
       loadCookie: null,
