@@ -76,7 +76,7 @@ class DiscoveryAPIHandler {
     });
   }
 
-  unblockResponses(responseText) {
+  unblockResponses() {
     throw new Error("You need to call blockNextResponses first!");
   }
 
@@ -567,7 +567,7 @@ add_task(async function discopane_no_cookies() {
   let requestPromise = new Promise(resolve => {
     amoServer.registerPathHandler("/discoapi", resolve);
   });
-  Services.cookies.add(
+  const cv = Services.cookies.add(
     AMO_TEST_HOST,
     "/",
     "name",
@@ -575,11 +575,12 @@ add_task(async function discopane_no_cookies() {
     false,
     false,
     false,
-    Date.now() / 1000 + 600,
+    Date.now() + 1000 * 600,
     {},
-    Ci.nsICookie.SAMESITE_NONE,
+    Ci.nsICookie.SAMESITE_UNSET,
     Ci.nsICookie.SCHEME_HTTP
   );
+  is(cv.result, Ci.nsICookieValidation.eOK, "Valid cookie");
   let win = await loadInitialView("discover");
   let request = await requestPromise;
   ok(!request.hasHeader("Cookie"), "discovery API should not receive cookies");
@@ -649,17 +650,24 @@ add_task(async function checkDiscopaneNotice() {
   });
 
   let win = await loadInitialView("extension");
-  let messageBar = win.document.querySelector("message-bar.discopane-notice");
+  let messageBar = win.document.querySelector(
+    "moz-message-bar.discopane-notice"
+  );
   ok(messageBar, "Recommended notice should exist in extensions view");
+  is(
+    messageBar.getAttribute("role"),
+    "alert",
+    "Recommended notice is an alert"
+  );
   await switchToDiscoView(win);
-  messageBar = win.document.querySelector("message-bar.discopane-notice");
+  messageBar = win.document.querySelector("moz-message-bar.discopane-notice");
   ok(messageBar, "Recommended notice should exist in disco view");
 
   messageBar.closeButton.click();
-  messageBar = win.document.querySelector("message-bar.discopane-notice");
+  messageBar = win.document.querySelector("moz-message-bar.discopane-notice");
   ok(!messageBar, "Recommended notice should not exist in disco view");
   await switchToNonDiscoView(win);
-  messageBar = win.document.querySelector("message-bar.discopane-notice");
+  messageBar = win.document.querySelector("moz-message-bar.discopane-notice");
   ok(!messageBar, "Recommended notice should not exist in extensions view");
 
   await closeView(win);
