@@ -18,9 +18,13 @@ const { XPCOMUtils } = ChromeUtils.importESModule(
 ChromeUtils.defineESModuleGetters(this, {
   UpdateUtils: "resource://gre/modules/UpdateUtils.sys.mjs",
   ctypes: "resource://gre/modules/ctypes.sys.mjs",
+  TelemetryArchiveTesting:
+    "resource://testing-common/TelemetryArchiveTesting.sys.mjs",
 });
 
 const PREF_APP_UPDATE_AUTO = "app.update.auto";
+const PREF_APP_UPDATE_BACKGROUND_ALLOWDOWNLOADSWITHOUTBITS =
+  "app.update.background.allowDownloadsWithoutBITS";
 const PREF_APP_UPDATE_BACKGROUNDERRORS = "app.update.backgroundErrors";
 const PREF_APP_UPDATE_BACKGROUNDMAXERRORS = "app.update.backgroundMaxErrors";
 const PREF_APP_UPDATE_BADGEWAITTIME = "app.update.badgeWaitTime";
@@ -71,14 +75,21 @@ const FILE_ACTIVE_UPDATE_XML = "active-update.xml";
 const FILE_ACTIVE_UPDATE_XML_TMP = "active-update.xml.tmp";
 const FILE_APPLICATION_INI = "application.ini";
 const FILE_BACKUP_UPDATE_CONFIG_JSON = "backup-update-config.json";
+const FILE_BACKUP_UPDATE_ELEVATED_LOG = "backup-update-elevated.log";
 const FILE_BACKUP_UPDATE_LOG = "backup-update.log";
-const FILE_BT_RESULT = "bt.result";
+const FILE_CHANNEL_PREFS =
+  AppConstants.platform == "macosx" ? "ChannelPrefs" : "channel-prefs.js";
+const FILE_INFO_PLIST = "Info.plist";
+const FILE_LAST_UPDATE_ELEVATED_LOG = "last-update-elevated.log";
 const FILE_LAST_UPDATE_LOG = "last-update.log";
 const FILE_PRECOMPLETE = "precomplete";
 const FILE_PRECOMPLETE_BAK = "precomplete.bak";
+const FILE_TEST_PROCESS_UPDATES = "test_process_updates.txt";
 const FILE_UPDATE_CONFIG_JSON = "update-config.json";
+const FILE_UPDATE_ELEVATED_LOG = "update-elevated.log";
 const FILE_UPDATE_LOG = "update.log";
 const FILE_UPDATE_MAR = "update.mar";
+const FILE_UPDATE_SETTINGS_FRAMEWORK = "UpdateSettings";
 const FILE_UPDATE_SETTINGS_INI = "update-settings.ini";
 const FILE_UPDATE_SETTINGS_INI_BAK = "update-settings.ini.bak";
 const FILE_UPDATE_STATUS = "update.status";
@@ -92,6 +103,22 @@ const UPDATE_SETTINGS_CONTENTS =
   "[Settings]\nACCEPTED_MAR_CHANNEL_IDS=xpcshell-test\n";
 const PRECOMPLETE_CONTENTS = 'rmdir "nonexistent_dir/"\n';
 
+const DIR_APP_INFO_PLIST_FILE_CONTENTS = `<?xml version="1.0" encoding="UTF-8"?>
+  <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+  <plist version="1.0">
+    <dict>
+      <key>CFBundleDevelopmentRegion</key><string>English</string>
+      <key>CFBundleDisplayName</key><string>dir</string>
+      <key>CFBundleExecutable</key><string>${AppConstants.MOZ_APP_NAME}</string>
+      <key>CFBundleIdentifier</key><string>${AppConstants.MOZ_MACBUNDLE_ID}</string>
+      <key>CFBundleInfoDictionaryVersion</key><string>6.0</string>
+      <key>CFBundleName</key><string>dir</string>
+      <key>CFBundlePackageType</key><string>APPL</string>
+      <key>CFBundleSignature</key><string>????</string>
+      <key>CFBundleVersion</key><string>1.0</string>
+    </dict>
+  </plist>`;
+
 const PR_RDWR = 0x04;
 const PR_CREATE_FILE = 0x08;
 const PR_TRUNCATE = 0x20;
@@ -125,14 +152,14 @@ XPCOMUtils.defineLazyServiceGetter(
   this,
   "gUpdateManager",
   "@mozilla.org/updates/update-manager;1",
-  "nsIUpdateManager"
+  Ci.nsIUpdateManager
 );
 
 XPCOMUtils.defineLazyServiceGetter(
   this,
   "gUpdateChecker",
   "@mozilla.org/updates/update-checker;1",
-  "nsIUpdateChecker"
+  Ci.nsIUpdateChecker
 );
 
 ChromeUtils.defineLazyGetter(this, "gDefaultPrefBranch", function test_gDPB() {
@@ -170,15 +197,26 @@ function waitForEvent(topic, status = null) {
 }
 
 /* Triggers post-update processing */
-function testPostUpdateProcessing() {
-  gAUS.observe(null, "test-post-update-processing", "");
+async function testPostUpdateProcessing() {
+  // All of the post update processing happens during initialization
+  await gAUS.internal.init(/* force = */ true);
 }
 
 /* Initializes the update service stub */
-function initUpdateServiceStub() {
-  Cc["@mozilla.org/updates/update-service-stub;1"].createInstance(
-    Ci.nsISupports
-  );
+async function initUpdateServiceStub() {
+  const updateServiceStub = Cc[
+    "@mozilla.org/updates/update-service-stub;1"
+  ].getService(Ci.nsIApplicationUpdateServiceStub);
+  await updateServiceStub.init();
+}
+
+/* Initializes the Update Service, even if it has already been initialized */
+async function reInitUpdateService() {
+  return gAUS.internal.init(/* force = */ true);
+}
+
+async function initUpdateService() {
+  return gAUS.init();
 }
 
 /**
@@ -189,11 +227,8 @@ function initUpdateServiceStub() {
  *         be reset. If false (the default), the update xml files will be read
  *         to populate the update metadata.
  */
-function reloadUpdateManagerData(skipFiles = false) {
-  let observeData = skipFiles ? "skip-files" : "";
-  gUpdateManager
-    .QueryInterface(Ci.nsIObserver)
-    .observe(null, "um-reload-update-data", observeData);
+async function reloadUpdateManagerData(skipFiles = false) {
+  await gUpdateManager.internal.reload(skipFiles);
 }
 
 const observer = {
@@ -220,6 +255,7 @@ function setUpdateChannel(aChannel) {
   debugDump(
     "setting default pref " + PREF_APP_UPDATE_CHANNEL + " to " + gChannel
   );
+  gDefaultPrefBranch.unlockPref(PREF_APP_UPDATE_CHANNEL);
   gDefaultPrefBranch.setCharPref(PREF_APP_UPDATE_CHANNEL, gChannel);
   gPrefRoot.addObserver(PREF_APP_UPDATE_CHANNEL, observer);
 }
@@ -323,6 +359,22 @@ function writeFile(aFile, aText) {
   fos.close();
 }
 
+/**
+ * Attempts to remove a file. Does not fail if the file does not exist.
+ *
+ * @param  file
+ *         The `nsIFile` to remove.
+ */
+function ensureRemoved(file) {
+  try {
+    file.remove(false);
+  } catch (ex) {
+    if (ex.result != Cr.NS_ERROR_FILE_NOT_FOUND) {
+      throw ex;
+    }
+  }
+}
+
 /**
  * Reads the current update operation/state in the status file in the patch
  * directory including the error code if it is present.
@@ -358,17 +410,6 @@ function readStatusFailedCode() {
   return readStatusFile().split(": ")[1];
 }
 
-/**
- * Returns whether or not applying the current update resulted in an error
- * verifying binary transparency information.
- *
- * @return true if there was an error result and false otherwise
- */
-function updateHasBinaryTransparencyErrorResult() {
-  let file = getUpdateDirFile(FILE_BT_RESULT);
-  return file.exists();
-}
-
 /**
  * Reads text from a file and returns the string.
  *
@@ -444,6 +485,7 @@ function getUpdateDirFile(aLeafName, aWhichDir = null) {
     case FILE_ACTIVE_UPDATE_XML_TMP:
     case FILE_UPDATE_CONFIG_JSON:
     case FILE_BACKUP_UPDATE_CONFIG_JSON:
+    case FILE_TEST_PROCESS_UPDATES:
     case FILE_UPDATE_TEST:
     case FILE_UPDATES_XML:
     case FILE_UPDATES_XML_TMP:
@@ -452,12 +494,14 @@ function getUpdateDirFile(aLeafName, aWhichDir = null) {
     case DIR_PATCH:
     case DIR_DOWNLOADING:
     case FILE_BACKUP_UPDATE_LOG:
+    case FILE_BACKUP_UPDATE_ELEVATED_LOG:
     case FILE_LAST_UPDATE_LOG:
+    case FILE_LAST_UPDATE_ELEVATED_LOG:
       file.append(DIR_UPDATES);
       file.append(aLeafName);
       return file;
-    case FILE_BT_RESULT:
     case FILE_UPDATE_LOG:
+    case FILE_UPDATE_ELEVATED_LOG:
     case FILE_UPDATE_MAR:
     case FILE_UPDATE_STATUS:
     case FILE_UPDATE_VERSION:
@@ -527,7 +571,6 @@ function removeUpdateFiles(aRemoveLogFiles) {
   let files = [
     [FILE_ACTIVE_UPDATE_XML],
     [FILE_UPDATES_XML],
-    [FILE_BT_RESULT],
     [FILE_UPDATE_STATUS],
     [FILE_UPDATE_VERSION],
     [FILE_UPDATE_MAR],
@@ -540,6 +583,9 @@ function removeUpdateFiles(aRemoveLogFiles) {
       [FILE_BACKUP_UPDATE_LOG],
       [FILE_LAST_UPDATE_LOG],
       [FILE_UPDATE_LOG],
+      [FILE_BACKUP_UPDATE_ELEVATED_LOG],
+      [FILE_LAST_UPDATE_ELEVATED_LOG],
+      [FILE_UPDATE_ELEVATED_LOG],
     ]);
   }
 
@@ -659,96 +705,6 @@ function getGREBinDir() {
   return Services.dirsvc.get(NS_GRE_BIN_DIR, Ci.nsIFile);
 }
 
-/**
- * Gets the unique mutex name for the installation.
- *
- * @return Global mutex path.
- * @throws If the function is called on a platform other than Windows.
- */
-function getPerInstallationMutexName() {
-  if (AppConstants.platform != "win") {
-    throw new Error("Windows only function called by a different platform!");
-  }
-
-  let hasher = Cc["@mozilla.org/security/hash;1"].createInstance(
-    Ci.nsICryptoHash
-  );
-  hasher.init(hasher.SHA1);
-
-  let exeFile = Services.dirsvc.get(XRE_EXECUTABLE_FILE, Ci.nsIFile);
-  let converter = Cc[
-    "@mozilla.org/intl/scriptableunicodeconverter"
-  ].createInstance(Ci.nsIScriptableUnicodeConverter);
-  converter.charset = "UTF-8";
-  let data = converter.convertToByteArray(exeFile.path.toLowerCase());
-
-  hasher.update(data, data.length);
-  return "Global\\MozillaUpdateMutex-" + hasher.finish(true);
-}
-
-/**
- * Closes a Win32 handle.
- *
- * @param  aHandle
- *         The handle to close.
- * @throws If the function is called on a platform other than Windows.
- */
-function closeHandle(aHandle) {
-  if (AppConstants.platform != "win") {
-    throw new Error("Windows only function called by a different platform!");
-  }
-
-  let lib = ctypes.open("kernel32.dll");
-  let CloseHandle = lib.declare(
-    "CloseHandle",
-    ctypes.winapi_abi,
-    ctypes.int32_t /* success */,
-    ctypes.void_t.ptr
-  ); /* handle */
-  CloseHandle(aHandle);
-  lib.close();
-}
-
-/**
- * Creates a mutex.
- *
- * @param  aName
- *         The name for the mutex.
- * @return The Win32 handle to the mutex.
- * @throws If the function is called on a platform other than Windows.
- */
-function createMutex(aName) {
-  if (AppConstants.platform != "win") {
-    throw new Error("Windows only function called by a different platform!");
-  }
-
-  const INITIAL_OWN = 1;
-  const ERROR_ALREADY_EXISTS = 0xb7;
-  let lib = ctypes.open("kernel32.dll");
-  let CreateMutexW = lib.declare(
-    "CreateMutexW",
-    ctypes.winapi_abi,
-    ctypes.void_t.ptr /* return handle */,
-    ctypes.void_t.ptr /* security attributes */,
-    ctypes.int32_t /* initial owner */,
-    ctypes.char16_t.ptr
-  ); /* name */
-
-  let handle = CreateMutexW(null, INITIAL_OWN, aName);
-  lib.close();
-  let alreadyExists = ctypes.winLastError == ERROR_ALREADY_EXISTS;
-  if (handle && !handle.isNull() && alreadyExists) {
-    closeHandle(handle);
-    handle = null;
-  }
-
-  if (handle && handle.isNull()) {
-    handle = null;
-  }
-
-  return handle;
-}
-
 /**
  * Synchronously writes the value of the app.update.auto setting to the update
  * configuration file on Windows or to a user preference on other platforms.
@@ -917,7 +873,7 @@ async function continueFileHandler(leafName) {
     "Waiting for file to be deleted, path: " + continueFile.path,
     interval,
     retries
-  ).catch(e => {
+  ).catch(_e => {
     logTestInfo(
       "Continue file was not removed after checking " +
         retries +
@@ -926,3 +882,102 @@ async function continueFileHandler(leafName) {
     );
   });
 }
+
+async function waitForUpdatePing(archiveChecker, expectedProperties) {
+  // We cannot control when the ping will be generated/archived after we trigger
+  // an update, so let's make sure to have one before moving on with validation.
+  let updatePing;
+  await TestUtils.waitForCondition(
+    async function () {
+      // Check that the ping made it into the Telemetry archive.
+      // The test data is defined in ../data/sharedUpdateXML.js
+      updatePing = await archiveChecker.promiseFindPing(
+        "update",
+        expectedProperties
+      );
+      return !!updatePing;
+    },
+    "Wait for Update Ping to be generated",
+    500,
+    100
+  );
+  return updatePing;
+}
+
+/**
+ * It's frequently desirable to run a test many times with different parameters
+ * each time.
+ *
+ * @param  testFn
+ *         The function to test. It is expected to be a function that takes a
+ *         single object of named parameters. For example:
+ *           function test({param1, param2})
+ * @param  parameters
+ *         This can either be an object or an array of objects.
+ *
+ *         If it is an array of objects, it will be treated as a list of
+ *         sets of parameters. The number of times the test is executed will be
+ *         equal to the length of the array.
+ *         For example:
+ *           [{param1: value1}, {param1: value2}]
+ *
+ *         If it is an object, each key will be treated as a parameter with the
+ *         corresponding value will be an array of values that parameter can
+ *         have. The number of times the test is executed will be equal to the
+ *         product of all the array lengths.
+ *         For example:
+ *            {param1: [value1, value2], {param2: [value3]}}
+ * @param  options
+ *         An additional object can be passed that with any of these supported
+ *         options:
+ *           skipFn
+ *             This is evaluated with the test's parameters before each test. If
+ *             it returns `true`, the test is not run with that set of
+ *             parameters.
+ */
+async function parameterizedTest(testFn, parameters, { skipFn } = {}) {
+  logTestInfo(`parameterizedTest - Testing ${testFn.name}`);
+
+  const maybeRunTest = async params => {
+    const invocationDesc = `${testFn.name}(${JSON.stringify(params)})`;
+    if (skipFn && (await skipFn(params))) {
+      logTestInfo("parameterizedTest - SKIPPING " + invocationDesc);
+      return;
+    }
+    logTestInfo("parameterizedTest - START " + invocationDesc);
+    await testFn(params);
+    logTestInfo("parameterizedTest - COMPLETE " + invocationDesc);
+  };
+
+  if (Array.isArray(parameters)) {
+    for (const params of parameters) {
+      await maybeRunTest(params);
+    }
+  } else {
+    const recurse = async (chosenParams, remainingPossibleParams) => {
+      if (!remainingPossibleParams.length) {
+        await maybeRunTest(chosenParams);
+        return;
+      }
+      const [param, argValues] = remainingPossibleParams.shift();
+      for (const argValue of argValues) {
+        chosenParams[param] = argValue;
+        // Clone parameters when we recurse.
+        await recurse(Object.assign({}, chosenParams), [
+          ...remainingPossibleParams,
+        ]);
+      }
+    };
+    await recurse({}, Object.entries(parameters));
+  }
+
+  logTestInfo(`parameterizedTest - Finished testing ${testFn.name}`);
+}
+
+async function startBitsMarDownload(url) {
+  return gAUS.wrappedJSObject.makeBitsRequest({ url });
+}
+
+async function connectToBitsMarDownload(bitsId) {
+  return gAUS.wrappedJSObject.makeBitsRequest({ bitsId });
+}
